@@ -31,18 +31,17 @@ import site
 import subprocess
 import sys
 from contextlib import contextmanager
-from typing import TYPE_CHECKING, Any, Callable, Iterator
+from typing import TYPE_CHECKING, Any, Callable, Iterator, TextIO
 
 from ansible_compat.config import ansible_version
 from ansible_compat.prerun import get_cache_dir
-from enrich.console import should_do_markup
 from filelock import FileLock, Timeout
 
 from ansiblelint import cli
 from ansiblelint._mockings import _perform_mockings_cleanup
 from ansiblelint.app import get_app
 from ansiblelint.color import console, console_options, reconfigure, render_yaml
-from ansiblelint.config import options
+from ansiblelint.config import get_version_warning, options
 from ansiblelint.constants import EXIT_CONTROL_C_RC, LOCK_TIMEOUT_RC
 from ansiblelint.file_utils import abspath, cwd, normpath
 from ansiblelint.skip_utils import normalize_tag
@@ -89,10 +88,6 @@ def initialize_options(arguments: list[str] | None = None) -> None:
     new_options = cli.get_config(arguments or [])
     new_options.cwd = pathlib.Path.cwd()
 
-    if new_options.version:
-        print(f"ansible-lint {__version__} using ansible {ansible_version()}")
-        sys.exit(0)
-
     if new_options.colored is None:
         new_options.colored = should_do_markup()
 
@@ -115,13 +110,17 @@ def initialize_options(arguments: list[str] | None = None) -> None:
     if not options.offline:
         options.cache_dir_lock = FileLock(f"{options.cache_dir}/.lock")
         try:
-            options.cache_dir_lock.acquire(timeout=120)
+            options.cache_dir_lock.acquire(timeout=180)
         except Timeout:
             _logger.error(
                 "Timeout waiting for another instance of ansible-lint to release the lock."
             )
             sys.exit(LOCK_TIMEOUT_RC)
 
+    # Avoid extra output noise from Ansible about using devel versions
+    if "ANSIBLE_DEVEL_WARNING" not in os.environ:
+        os.environ["ANSIBLE_DEVEL_WARNING"] = "false"
+
 
 def _do_list(rules: RulesCollection) -> int:
     # On purpose lazy-imports to avoid pre-loading Ansible
@@ -133,7 +132,7 @@ def _do_list(rules: RulesCollection) -> int:
         rules_as_str,
     )
 
-    if options.listrules:
+    if options.list_rules:
 
         _rule_format_map: dict[str, Callable[..., Any]] = {
             "plain": rules_as_str,
@@ -145,8 +144,8 @@ def _do_list(rules: RulesCollection) -> int:
         console.print(_rule_format_map[options.format](rules), highlight=False)
         return 0
 
-    if options.listtags:
-        console.print(render_yaml(rules.listtags()))
+    if options.list_tags:
+        console.print(render_yaml(rules.list_tags()))
         return 0
 
     # we should not get here!
@@ -171,7 +170,7 @@ def _do_transform(result: LintResult, opts: Namespace) -> None:
     transformer.run()
 
 
-# pylint: disable=too-many-branches
+# pylint: disable=too-many-branches,too-many-statements
 def main(argv: list[str] | None = None) -> int:  # noqa: C901
     """Linter CLI entry point."""
     # alter PATH if needed (venv support)
@@ -184,26 +183,44 @@ def main(argv: list[str] | None = None) -> int:  # noqa: C901
     console_options["force_terminal"] = options.colored
     reconfigure(console_options)
 
+    if options.version:
+        console.print(
+            f"ansible-lint [repr.number]{__version__}[/] using ansible [repr.number]{ansible_version()}[/]"
+        )
+        msg = get_version_warning()
+        if msg:
+            console.print(msg)
+        sys.exit(0)
+
     initialize_logger(options.verbosity)
     _logger.debug("Options: %s", options)
     _logger.debug(os.getcwd())
 
-    app = get_app(offline=options.offline)
+    if not options.offline:
+        # refresh schemas must happen before loading rules
+        if "ansiblelint.schemas" in sys.modules:
+            raise RuntimeError("ansiblelint.schemas should not be loaded yet")
+        # pylint: disable=import-outside-toplevel
+        from ansiblelint.schemas import refresh_schemas
+
+        refresh_schemas()
+
     # pylint: disable=import-outside-toplevel
     from ansiblelint.rules import RulesCollection
     from ansiblelint.runner import _get_matches
 
-    rules = RulesCollection(options.rulesdirs, profile=options.profile)
+    rules = RulesCollection(options.rulesdirs, profile_name=options.profile)
 
-    if options.profile == []:
+    if options.list_profiles:
         from ansiblelint.generate_docs import profiles_as_rich
 
         console.print(profiles_as_rich())
         return 0
 
-    if options.listrules or options.listtags:
+    if options.list_rules or options.list_tags:
         return _do_list(rules)
 
+    app = get_app()
     if isinstance(options.tags, str):
         options.tags = options.tags.split(",")
     result = _get_matches(rules, options)
@@ -211,8 +228,9 @@ def main(argv: list[str] | None = None) -> int:  # noqa: C901
     if options.write_list:
         _do_transform(result, options)
 
-    mark_as_success = False
+    mark_as_success = True
     if result.matches and options.progressive:
+        mark_as_success = False
         _logger.info(
             "Matches found, running again on previous revision in order to detect regressions"
         )
@@ -244,6 +262,9 @@ def main(argv: list[str] | None = None) -> int:  # noqa: C901
                     ignored,
                 )
 
+    if options.strict and result.matches:
+        mark_as_success = False
+
     app.render_matches(result.matches)
 
     _perform_mockings_cleanup()
@@ -324,6 +345,19 @@ def path_inject() -> None:
     # This must be run before we do run any subprocesses, and loading config
     # does this as part of the ansible detection.
     paths = [x for x in os.environ.get("PATH", "").split(os.pathsep) if x]
+
+    # Expand ~ in PATH as it known to break many tools
+    expanded = False
+    for idx, path in enumerate(paths):
+        if "~" in path:
+            paths[idx] = os.path.expanduser(path)
+            expanded = True
+    if expanded:
+        print(
+            "WARNING: PATH altered to expand ~ in it. Read https://stackoverflow.com/a/44704799/99834 and correct your system configuration.",
+            file=sys.stderr,
+        )
+
     inject_paths = []
 
     userbase_bin_path = f"{site.getuserbase()}/bin"
@@ -341,6 +375,7 @@ def path_inject() -> None:
             f"WARNING: PATH altered to include {', '.join(inject_paths)} :: This is usually a sign of broken local setup, which can cause unexpected behaviors.",
             file=sys.stderr,
         )
+    if inject_paths or expanded:
         os.environ["PATH"] = os.pathsep.join([*inject_paths, *paths])
 
     # We do know that finding ansible in PATH does not guarantee that it is
@@ -350,5 +385,50 @@ def path_inject() -> None:
         raise RuntimeError("Failed to find ansible executable in PATH")
 
 
+# Based on Ansible implementation
+def to_bool(value: Any) -> bool:
+    """Return a bool for the arg."""
+    if value is None or isinstance(value, bool):
+        return bool(value)
+    if isinstance(value, str):
+        value = value.lower()
+    if value in ("yes", "on", "1", "true", 1):
+        return True
+    return False
+
+
+def should_do_markup(stream: TextIO = sys.stdout) -> bool:
+    """Decide about use of ANSI colors."""
+    py_colors = None
+
+    # https://xkcd.com/927/
+    for env_var in ["PY_COLORS", "CLICOLOR", "FORCE_COLOR", "ANSIBLE_FORCE_COLOR"]:
+        value = os.environ.get(env_var, None)
+        if value is not None:
+            py_colors = to_bool(value)
+            break
+
+    # If deliberately disabled colors
+    if os.environ.get("NO_COLOR", None):
+        return False
+
+    # User configuration requested colors
+    if py_colors is not None:
+        return to_bool(py_colors)
+
+    term = os.environ.get("TERM", "")
+    if "xterm" in term:
+        return True
+
+    if term == "dumb":
+        return False
+
+    # Use tty detection logic as last resort because there are numerous
+    # factors that can make isatty return a misleading value, including:
+    # - stdin.isatty() is the only one returning true, even on a real terminal
+    # - stderr returning false if user user uses a error stream coloring solution
+    return stream.isatty()
+
+
 if __name__ == "__main__":
     _run_cli_entrypoint()
