@@ -37,6 +37,7 @@ DEFAULT_WARN_LIST = [
     "name[play]",
     "role-name",
     "warning[empty-playbook]",  # because ansible considers it warning only
+    "role-name[path]",  # too new
 ]
 
 DEFAULT_KINDS = [
@@ -95,25 +96,6 @@ BASE_KINDS = [
     {"text/yaml": "**/.*.{yaml,yml}"},
 ]
 
-
-# Maps kinds to JSON schemas
-# See https://www.schemastore.org/json/
-JSON_SCHEMAS = {
-    # Do not use anchors in these URLs because python-jsonschema does not support them:
-    "playbook": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-playbook.json",
-    "tasks": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-tasks.json",
-    "vars": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-vars.json",
-    "requirements": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-requirements.json",
-    "meta": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-meta.json",
-    "galaxy": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-galaxy.json",
-    "execution-environment": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-ee.json",
-    "meta-runtime": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-meta-runtime.json",
-    "inventory": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-inventory.json",
-    "ansible-lint-config": "https://raw.githubusercontent.com/ansible/ansible-lint/main/src/ansiblelint/schemas/ansible-lint-config.json",
-    "ansible-navigator-config": "https://raw.githubusercontent.com/ansible/ansible-navigator/main/src/ansible_navigator/data/ansible-navigator.json",
-    "arg_specs": "https://raw.githubusercontent.com/ansible/schemas/main/f/ansible-argument-specs.json",
-}
-
 PROFILES = yaml_from_file(Path(__file__).parent / "data" / "profiles.yml")
 
 options = Namespace(
@@ -125,8 +107,8 @@ options = Namespace(
     exclude_paths=[],
     format="rich",
     lintables=[],
-    listrules=False,
-    listtags=False,
+    list_rules=False,
+    list_tags=False,
     write_list=[],
     parseable=False,
     quiet=False,
@@ -140,6 +122,7 @@ options = Namespace(
     mock_modules=[],
     mock_roles=[],
     loop_var_prefix=None,
+    only_builtins_allow_collections=[],
     var_naming_pattern=None,
     offline=False,
     project_dir=".",  # default should be valid folder (do not use None here)
@@ -148,6 +131,7 @@ options = Namespace(
     skip_action_validation=True,
     strict=False,
     rules={},  # Placeholder to set and keep configurations for each rule.
+    profile=None,
 )
 
 # Used to store detected tag deprecations
@@ -224,7 +208,6 @@ def guess_install_method() -> str:
         with warnings.catch_warnings(record=True):
             warnings.simplefilter("always")
             # pylint: disable=import-outside-toplevel
-            from pip._internal.exceptions import UninstallationError
             from pip._internal.metadata import get_default_environment
             from pip._internal.req.req_uninstall import uninstallation_paths
 
@@ -236,7 +219,8 @@ def guess_install_method() -> str:
             else:
                 logging.debug("Skipping %s as it is not installed.", package_name)
                 use_pip = False
-    except (UninstallationError, AttributeError, ImportError) as exc:
+    # pylint: disable=broad-except
+    except Exception as exc:
         # On Fedora 36, we got a AttributeError exception from pip that we want to avoid
         logging.debug(exc)
         use_pip = False
@@ -247,9 +231,14 @@ def guess_install_method() -> str:
 
 def get_version_warning() -> str:
     """Display warning if current version is outdated."""
+    # 0.1dev1 is special fallback version
+    if __version__ == "0.1.dev1":
+        return ""
+
     msg = ""
     data = {}
     current_version = Version(__version__)
+
     if not os.path.exists(CACHE_DIR):
         os.makedirs(CACHE_DIR)
     cache_file = f"{CACHE_DIR}/latest.json"
@@ -278,15 +267,14 @@ def get_version_warning() -> str:
 
     html_url = data["html_url"]
     new_version = Version(data["tag_name"][1:])  # removing v prefix from tag
-    # breakpoint()
 
     if current_version > new_version:
         msg = "[dim]You are using a pre-release version of ansible-lint.[/]"
     elif current_version < new_version:
         msg = f"""[warning]A new release of ansible-lint is available: [red]{current_version}[/] → [green][link={html_url}]{new_version}[/][/][/]"""
 
-    pip = guess_install_method()
-    if pip:
-        msg += f" Upgrade by running: [info]{pip}[/]"
+        pip = guess_install_method()
+        if pip:
+            msg += f" Upgrade by running: [info]{pip}[/]"
 
     return msg
