@@ -29,6 +29,24 @@ KEYWORDS_WITH_IMPLICIT_TEMPLATE = ("changed_when", "failed_when", "until", "when
 
 Token = namedtuple("Token", "lineno token_type value")
 
+DEANNOTATE_KEYS = ("__file__", "__line__", "__start_line__", "__end_line__")
+
+
+def deannotate(data: Any) -> Any:
+    """Remove our annotations like __file__ and __line__ and return a JSON serializable object."""
+    if isinstance(data, dict):
+        result = data.copy()
+        for key, value in data.items():
+            if key in DEANNOTATE_KEYS:
+                del result[key]
+            elif isinstance(value, list):
+                for i, item in enumerate(value):
+                    value[i] = deannotate(item)
+            elif isinstance(value, dict):
+                result[key] = deannotate(value)
+        return result
+    return result
+
 
 class JinjaRule(AnsibleLintRule):
     """Rule that looks inside jinja2 templates."""
@@ -62,7 +80,7 @@ class JinjaRule(AnsibleLintRule):
                     template(
                         basedir=file.dir if file else ".",
                         value=v,
-                        variables={},
+                        variables=deannotate(task.get("vars", {})),
                         fail_on_error=True,  # we later decide which ones to ignore or not
                     )
                 # ValueError RepresenterError
@@ -103,7 +121,6 @@ class JinjaRule(AnsibleLintRule):
                             _logger.debug("Ignored AnsibleError: %s", exc)
                             bypass = True
                         else:
-                            # breakpoint()
                             bypass = False
                     elif re.match(r"^lookup plugin (.*) not found$", exc.message):
                         # lookup plugin 'template' not found
@@ -120,7 +137,9 @@ class JinjaRule(AnsibleLintRule):
                             tag=f"{self.id}[invalid]",
                         )
 
-                reformatted, details, tag = self.check_whitespace(v, key=key)
+                reformatted, details, tag = self.check_whitespace(
+                    v, key=key, lintable=file
+                )
                 if reformatted != v:
                     return self.create_matcherror(
                         message=self._msg(tag=tag, value=v, reformatted=reformatted),
@@ -142,7 +161,9 @@ class JinjaRule(AnsibleLintRule):
             # pylint: disable=unused-variable
             for key, v, path in nested_items_path(data):
                 if isinstance(v, AnsibleUnicode):
-                    reformatted, details, tag = self.check_whitespace(v, key=key)
+                    reformatted, details, tag = self.check_whitespace(
+                        v, key=key, lintable=file
+                    )
                     if reformatted != v:
                         results.append(
                             self.create_matcherror(
@@ -179,7 +200,7 @@ class JinjaRule(AnsibleLintRule):
         new_text = self.unlex(tokens)
         if text != new_text:
             _logger.debug(
-                "Unable to perform full roundrip lex-unlex on jinja template (expected when '-' modifier is used): {text} -> {new_text}"
+                "Unable to perform full roundtrip lex-unlex on jinja template (expected when '-' modifier is used): {text} -> {new_text}"
             )
         return tokens
 
@@ -198,7 +219,7 @@ class JinjaRule(AnsibleLintRule):
 
     # pylint: disable=too-many-branches,too-many-statements,too-many-locals
     def check_whitespace(  # noqa: max-complexity: 13
-        self, text: str, key: str
+        self, text: str, key: str, lintable: Lintable | None = None
     ) -> tuple[str, str, str]:
         """Check spacing inside given jinja2 template string.
 
@@ -229,7 +250,15 @@ class JinjaRule(AnsibleLintRule):
         implicit = False
 
         # implicit templates do not have the {{ }} wrapping
-        if key in KEYWORDS_WITH_IMPLICIT_TEMPLATE:
+        if (
+            key in KEYWORDS_WITH_IMPLICIT_TEMPLATE
+            and lintable
+            and lintable.kind
+            in (
+                "playbook",
+                "task",
+            )
+        ):
             implicit = True
             text = cook(text, implicit=implicit)
 
@@ -279,6 +308,8 @@ class JinjaRule(AnsibleLintRule):
 
         except jinja2.exceptions.TemplateSyntaxError as exc:
             return "", str(exc.message), "invalid"
+        # https://github.com/PyCQA/pylint/issues/7433 - py311 only
+        # pylint: disable=c-extension-no-member
         except (NotImplementedError, black.parsing.InvalidInput) as exc:
             # black is not able to recognize all valid jinja2 templates, so we
             # just ignore InvalidInput errors.
@@ -544,7 +575,9 @@ if "pytest" in sys.modules:  # noqa: C901
         """Tests our ability to spot spacing errors inside jinja2 templates."""
         rule = JinjaRule()
 
-        reformatted, details, returned_tag = rule.check_whitespace(text, key="name")
+        reformatted, details, returned_tag = rule.check_whitespace(
+            text, key="name", lintable=Lintable("playbook.yml")
+        )
         assert tag == returned_tag, details
         assert expected == reformatted
 
@@ -571,10 +604,29 @@ if "pytest" in sys.modules:  # noqa: C901
     def test_jinja_implicit(text: str, expected: str, tag: str) -> None:
         """Tests our ability to spot spacing errors implicit jinja2 templates."""
         rule = JinjaRule()
-        reformatted, details, returned_tag = rule.check_whitespace(text, key="when")
+        # implicit jinja2 are working only inside playbooks and tasks
+        lintable = Lintable(name="playbook.yml", kind="playbook")
+        reformatted, details, returned_tag = rule.check_whitespace(
+            text, key="when", lintable=lintable
+        )
         assert tag == returned_tag, details
         assert expected == reformatted
 
+    @pytest.mark.parametrize(
+        ("lintable", "matches"),
+        (pytest.param("examples/playbooks/vars/rule_jinja_vars.yml", 0, id="0"),),
+    )
+    def test_jinja_file(lintable: str, matches: int) -> None:
+        """Tests our ability to process var filesspot spacing errors."""
+        collection = RulesCollection()
+        collection.register(JinjaRule())
+        errs = Runner(lintable, rules=collection).run()
+        assert len(errs) == matches
+        for err in errs:
+            assert isinstance(err, JinjaRule)
+            assert errs[0].tag == "jinja[invalid]"
+            assert errs[0].rule.id == "jinja"
+
     def test_jinja_invalid() -> None:
         """Tests our ability to spot spacing errors inside jinja2 templates."""
         collection = RulesCollection()
