@@ -182,7 +182,7 @@ def _playbook_items(pb_data: AnsibleBaseYAMLObject) -> ItemsView:  # type: ignor
 
     # "if play" prevents failure if the play sequence contains None,
     # which is weird but currently allowed by Ansible
-    # https://github.com/ansible-community/ansible-lint/issues/849
+    # https://github.com/ansible/ansible-lint/issues/849
     return [item for play in pb_data if play for item in play.items()]  # type: ignore
 
 
@@ -472,7 +472,7 @@ def _rolepath(basedir: str, role: str) -> str | None:
         path_dwim(basedir, os.path.join("..", role)),
     ]
 
-    for loc in get_app(offline=True).runtime.config.default_roles_path:
+    for loc in get_app().runtime.config.default_roles_path:
         loc = os.path.expanduser(loc)
         possible_paths.append(path_dwim(loc, role))
 
